////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// audio.cpp
////////////////////////////////////////////////////////////////////////////////

#include "audio.h"

////////////////////////////////////////////////////////////////////////////////
// AudioManager handles the details of loading various types of audio files.
// Only a title for the piece and the file path are needed.
////////////////////////////////////////////////////////////////////////////////

AudioManager *AudioManager::mInstance = 0;

// Ensures correct FMOD version exists and initializes FMOD sound system.
AudioManager::AudioManager()
{
	if (FSOUND_GetVersion() < FMOD_VERSION)
	{
		printf("Error: Wrong DLL version. You should be using FMOD %.02f\n", FMOD_VERSION);
	}

	if(!FSOUND_Init(44100, 32, 0))
	{
		printf("Could not initialize the FMOD sound system.\n");
	}

	// initialize all the supported extensions
	mModuleExtensions.push_back(".fsb");
	mModuleExtensions.push_back(".it");
	mModuleExtensions.push_back(".mid");
	mModuleExtensions.push_back(".mod");
	mModuleExtensions.push_back(".rmi");
	mModuleExtensions.push_back(".s3m");
	mModuleExtensions.push_back(".sgt");
	mModuleExtensions.push_back(".xm");

	mSampleExtensions.push_back(".mp2");
	mSampleExtensions.push_back(".mp3");
	mSampleExtensions.push_back(".ogg");
	mSampleExtensions.push_back(".raw");
	mSampleExtensions.push_back(".wav");
}

// Frees memory used by audio files and shuts down FMOD sound system.
AudioManager::~AudioManager()
{
	// frees memory associated with each audio file
	std::map<std::string, AudioFile *>::iterator i = mMusicList.begin();
	while(i != mMusicList.end())
	{
		delete i->second;
		++i;
	}

	mModuleExtensions.clear();
	mSampleExtensions.clear();
	mMusicList.clear();
		
	FSOUND_Close();
	mInstance = NULL;
}

// Returns the only instance of this class, if any.
AudioManager *AudioManager::Instance()
{
	if(0 == mInstance)
	{
		mInstance = new AudioManager();
	}
	return mInstance;
}

// Determines what kind of file to create based on the file extension and creates it.
void AudioManager::load(std::string title, std::string fileName)
{
	unsigned int i = 0;
	//bool songTitleExists = false;

	// ensure that a song with the same title does not already exist in the list
	std::map<std::string, AudioFile *>::iterator it = mMusicList.begin();
	while(it != mMusicList.end())
	{
		if(it->first == title.c_str())
		{
			return;
		}
		++it;
	}

	// if the file extension matches a module type, the match is loaded
	for(i = 0; i < mModuleExtensions.size(); ++i)
	{
		if(fileName.rfind(mModuleExtensions[i]) == fileName.length() - mModuleExtensions[i].length())
		{
			mMusicList[title] = new Module(fileName);
			return;
		}
	}

	// if the file extension matches a sample type, the match is loaded
	for(i = 0; i < mSampleExtensions.size(); ++i)
	{
		if(fileName.rfind(mSampleExtensions[i]) == fileName.length() - mSampleExtensions[i].length())
		{
			mMusicList[title] = new Sample(fileName);
			return;
		}
	}
}

// Frees the appropriate audio file and removes it from the playable list
void AudioManager::unload(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		delete mMusicList[title];

		mMusicList.erase(title);
	}
	else
	{
		printf("Error: Attempted to unload nonexistant title.");
	}
}

// Plays the audio file with the specified title.
void AudioManager::play(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		mMusicList[title]->play();
	}
	else
	{
		printf("Error: Attempted to play nonexistant title.");
	}
}

// Pauses the audio file with the specified title.
void AudioManager::pause(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		mMusicList[title]->pause();
	}
	else
	{
		printf("Error: Attempted to pause nonexistant title.");
	}
}

// Pauses the audio file with the specified title.
bool AudioManager::isPaused(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		return mMusicList[title]->isPaused();
	}
	else
	{
		printf("Error: Attempted to check status of nonexistant title.");
		return false;
	}
}

// Resumes the paused audio file with the specified title.
void AudioManager::resume(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		mMusicList[title]->resume();
	}
	else
	{
		printf("Error: Attempted to pause nonexistant title.");
	}
}

// Stops the audio file with the specified title.
void AudioManager::stop(std::string title)
{
	if(0 != mMusicList.count(title))
	{
		mMusicList[title]->stop();
	}
	else
	{
		printf("Error: Attempted to stop nonexistant title.");
	}
}

// Sets the volume of the audio file with the specified title.
void AudioManager::setVolume(std::string title, int volume)
{
	if((0 != mMusicList.count(title)) && (volume > 0))
	{
		mMusicList[title]->setVolume(volume);
	}
	else
	{
		printf("Error: Attempted to set volume of nonexistant title.");
	}
}

////////////////////////////////////////////////////////////////////////////////
// Module supports sequenced music files like midi and mod.
////////////////////////////////////////////////////////////////////////////////

Module::Module(std::string & fileName)
{
	mFileName = fileName;
	mModule = FMUSIC_LoadSong(fileName.c_str());
}

Module::~Module()
{
	FMUSIC_FreeSong(mModule);
}

void Module::play()
{
	FMUSIC_PlaySong(mModule);
}

void Module::pause()
{
	FMUSIC_SetPaused(mModule, true);
}

bool Module::isPaused()
{
	return FMUSIC_GetPaused(mModule);
}

void Module::resume()
{
	FMUSIC_SetPaused(mModule, false);
}

void Module::stop()
{
	FMUSIC_StopSong(mModule);
}

void Module::setVolume(int volume)
{
	FMUSIC_SetMasterVolume(mModule, volume);
}

////////////////////////////////////////////////////////////////////////////////
// Sample supports sampled music like .mp3 and .wav files.
////////////////////////////////////////////////////////////////////////////////

Sample::Sample(std::string & fileName)
{
	mFileName = fileName;
	mSample = FSOUND_Sample_Load(FSOUND_FREE, fileName.c_str(), FSOUND_NORMAL | FSOUND_LOOP_OFF, 0, 0);
}

Sample::~Sample()
{
	FSOUND_Sample_Free(mSample);
}

void Sample::play()
{
	mChannel = FSOUND_PlaySound(FSOUND_FREE, mSample);
}

void Sample::pause()
{
	FSOUND_SetPaused(mChannel, true);
}

void Sample::resume()
{
	FSOUND_SetPaused(mChannel, false);
}

bool Sample::isPaused()
{
	return FSOUND_GetPaused(mChannel);
}

void Sample::stop()
{
	FSOUND_StopSound(mChannel);
}

void Sample::setVolume(int volume)
{
	FSOUND_SetSFXMasterVolume(volume);
}
