////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// framework.cpp
////////////////////////////////////////////////////////////////////////////////

#include "framework.h"

////////////////////////////////////////////////////////////////////////////////
// GameFramework provides common game functionality to speed development time.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
GameFramework *GameFramework::mInstance = 0;

// Returns the only instance of the class, creating it if it doesn't exist yet.
GameFramework *GameFramework::Instance()
{
	if(0 == mInstance)
	{
		mInstance = new GameFramework;
	}
	return mInstance;
}

// Sets up a window with specified parameters and transitions to first game state.
void GameFramework::Initialize(int *argc, char **argv, GameState *state,
							   const char *title, int width, int height, bool fullscreen)
{
	glutInit(argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH | GLUT_STENCIL);
	glutInitWindowPosition(4, 23);
	glutInitWindowSize(width, height);
	glutCreateWindow(title);
	// set callbacks and create display list for font
	SetCallbacks();
	SetUpFont();
	// default to hidden surface removal, smooth shading, and nice perspective
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glEnable(GL_STENCIL_TEST);
	glDepthFunc(GL_LEQUAL);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	// ignore repeat keystrokes from holding a key down
	glutIgnoreKeyRepeat(GLUT_KEY_REPEAT_ON);
	// set background to blue
	glClearColor(0.0, 0.0, 1.0, 1.0);

	// load fullscreen, FPS-display, and difficulty settings from file
	LoadSettingsFromFile();

	// clear key buffer
	memset(mKeyBuffer, 0, sizeof(mKeyBuffer));

	SetState(state);

	glutMainLoop();
}

// Cleans up current state and initializes the new state.
void GameFramework::SetState(GameState *state)
{
	if(NULL != mCurrentState)
	{
		mCurrentState->CleanUp();
	}
	mCurrentState = state;
	mCurrentState->Initialize();
}

// Deletes the font display list and exits.
void GameFramework::Shutdown()
{
	// free font display list memory
	glDeleteLists(mTextBase, FONT_ASCII_VALUE_LIMIT);

	// save the preferences of the user.
	WriteSettingsToFile();

	// cleans up current state
	if(NULL != mCurrentState)
	{
		mCurrentState->CleanUp();
	}
	mCurrentState = NULL;

	GameFramework::~GameFramework();

	exit(0);
}

// Sets OpenGL callback functions.
void GameFramework::SetCallbacks()
{
	glutReshapeFunc(Reshape);
	glutKeyboardFunc(HandleInput);
	glutKeyboardUpFunc(KeyUp);
	glutSpecialFunc(HandleInputSpecial);
	glutSpecialUpFunc(KeyUpSpecial);
	glutDisplayFunc(Render);
	glutIdleFunc(Update);
}

// Handles the resizing of the window and sets up the perspective projection.
void GameFramework::Reshape(int width, int height)
{
	// snap the width and height to multiples of 4 and 3, respectively
	width -= (width % WINDOW_WIDTH_RATIO);
	height -= (height % WINDOW_HEIGHT_RATIO);

	// make sure the window does not get sized too small
	// because the menu text looks bad
	if(height < MIN_WINDOW_HEIGHT)
	{
		height = MIN_WINDOW_HEIGHT;
	}
	if(width < MIN_WINDOW_WIDTH)
	{
		width = MIN_WINDOW_WIDTH;
	}

	// ensure the ratio of width to height is 4 to 3
	// by making the dimension that is too small larger.
	if(height < (WINDOW_HEIGHT_RATIO * width / WINDOW_WIDTH_RATIO) )
	{
		height = WINDOW_HEIGHT_RATIO * width / WINDOW_WIDTH_RATIO;
	}
	if(width <= (WINDOW_WIDTH_RATIO * height / WINDOW_HEIGHT_RATIO))
	{
		width = WINDOW_WIDTH_RATIO * height / WINDOW_HEIGHT_RATIO;
	}

	glutReshapeWindow(width, height);
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)width/height, 0.1, 100.0);
}

// Handles buffered keyboard input of keys with corresponding ASCII key codes.
void GameFramework::HandleInput(unsigned char key, int x, int y)
{
	mInstance->mKeyBuffer[key] = 1;
	mInstance->mCurrentState->HandleInput(key);
}

// Handles key releases for immediate keyboard data for keys with ASCII codes.
void GameFramework::KeyUp(unsigned char key, int x, int y)
{
	mInstance->mKeyBuffer[key] = 0;
}

// Handles buffered keyboard input of keys without corresponding ASCII key codes.
void GameFramework::HandleInputSpecial(int key, int x, int y)
{
	mInstance->mKeyBuffer[(unsigned char)key] = 1;
	mInstance->mCurrentState->HandleInput(key);
}

// Handles key releases for immediate keyboard data for keys without ASCII codes.
void GameFramework::KeyUpSpecial(int key, int x, int y)
{
	mInstance->mKeyBuffer[(unsigned char)key] = 0;
}

// Renders a frame of the current state.
void GameFramework::Render()
{
	mInstance->mCurrentState->Render();
	glutSwapBuffers();
}

// Updates the timer for FPS calculation, allows states to update themselves
// and handle immediate keyboard input for both ASCII and special characters,
// and renders the next frame.
void GameFramework::Update()
{
	mInstance->mTimer.Update();
	mInstance->mCurrentState->Update(mInstance->mTimer.GetDelta(), mInstance->mKeyBuffer);
	glutPostRedisplay();
	Sleep(1);
}

// Load the settings from file, "assets\text\settings.txt"
void GameFramework::LoadSettingsFromFile()
{
	std::fstream settings;
	settings.open("assets\\text\\settings.txt", std::ios::in);

	if(settings.is_open())
	{
		settings >> mSharedData[Fullscreen];
		settings >> mSharedData[DisplayFPS];
		settings >> mSharedData[Difficulty];
		settings.close();
	}

	ValidateSettingsFileData();
	UpdateFullscreenStatus();
}

// Write the settings to file, "assets\text\settings.txt"
void GameFramework::WriteSettingsToFile()
{
	ValidateSettingsFileData();

	std::fstream settings;
	settings.open("assets\\text\\settings.txt", std::ios::out);

	if(settings.is_open())
	{
		settings << mSharedData[Fullscreen] << '\n';
		settings << mSharedData[DisplayFPS] << '\n';
		settings << mSharedData[Difficulty] << '\n';
		settings.close();
	}
}

// Make sure the shared data in the settings file is within the correct range.
void GameFramework::ValidateSettingsFileData()
{
	assert(mSharedData[Fullscreen] >= 0 && mSharedData[Fullscreen] <= 1);
	assert(mSharedData[DisplayFPS] >= 0 && mSharedData[DisplayFPS] <= 1);
	assert(mSharedData[Difficulty] >= 0 && mSharedData[Difficulty] <= 2);
}

// 
int GameFramework::GetSharedData(int index)
{
	return mSharedData[index];
}

// 
void GameFramework::SetSharedData(int index, int value)
{
	mSharedData[index] = value;
}

// Changes the program to run fullscreen or windowed based on the current settings.
void GameFramework::UpdateFullscreenStatus()
{
	if(mSharedData[Fullscreen])
	{
		glutFullScreen();
	}
	else
	{
		glutReshapeWindow(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
		// the offset of (4, 23) pixels accounts for a standard window border
		glutPositionWindow(4, 23);
	}
	WriteSettingsToFile();
}

// Sets up a display list for a GLUT font. For other font options
// besides GLUT_BITMAP_TIMES_ROMAN_24, check glut.h
void GameFramework::SetUpFont()
{
	mTextBase = glGenLists(FONT_ASCII_VALUE_LIMIT);

	for(int i = 0; i < FONT_ASCII_VALUE_LIMIT; i++)
	{
		glNewList(mTextBase + i, GL_COMPILE);
		glutBitmapCharacter(GLUT_BITMAP_TIMES_ROMAN_24, i);
		glEndList();
	}

	glListBase(mTextBase);
}

// Draw the specified text at a given position. The bottom left of the text is
// positioned at window coordinates (x, y) where (0.0, 0.0) is the bottom left
// of the window and (1.0, 1.0) is at the top right of the window.
// This method was previously a member of the Text class.
void GameFramework::DrawText(std::string text, float x, float y, bool centered)
{
	// calculate new x if text should be centered
	if(centered)
	{
		int stringWidth = 0;
		int i = 0;

		for(i = 0; i < text.length(); i++)
		{
			stringWidth += glutBitmapWidth(GLUT_BITMAP_TIMES_ROMAN_24, text[i]);
		}
		x = (1.0 - ((float)stringWidth + 1)/glutGet(GLUT_WINDOW_WIDTH)) / 2.0;
	}

	// change to 2d mode
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0.0, 1.0, 0.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// position in lower left corner and draw text
	glRasterPos2f(x, y);
	glCallLists(text.length(), GL_UNSIGNED_BYTE, text.c_str());

	// change back to perspective mode
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
}

// Displays frames per second (average of last 20 frames) in lower left corner.
void GameFramework::DrawFPS()
{
	static int mFpsAvgPos;
	static int mFpsAvg[FRAMES_TO_CALC_AVERAGE_FPS];

	if(mSharedData[DisplayFPS])
	{
		int i = 0;
		int j = 0;
		int fpsAverage = 0;
		char fpsChars[6];
		std::string fpsString;

		mFpsAvg[mFpsAvgPos] = (int)(1.0 / mTimer.GetDelta());
		++mFpsAvgPos;
		if(mFpsAvgPos > FRAMES_TO_CALC_AVERAGE_FPS)
			mFpsAvgPos = 0;
		for(j = 0; j < FRAMES_TO_CALC_AVERAGE_FPS; ++j)
		{
			fpsAverage += mFpsAvg[j];
		}

		_itoa_s((int)((float)fpsAverage / FRAMES_TO_CALC_AVERAGE_FPS), fpsChars, 10);

		fpsString = "FPS: ";
		fpsString.append(fpsChars);
	
		mInstance->DrawText(fpsString, 0.01, 0.01);
	}
}
