////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// framework.h
////////////////////////////////////////////////////////////////////////////////

#ifndef FRAMEWORK_H
#define FRAMEWORK_H

#include <fstream>
#include <map>
#include <string>
#include <vector>
#include <GL/glut.h>
#include "types.h"
#include "timer.h"
#include "menu.h"

const int DEFAULT_WINDOW_WIDTH = 800;
const int DEFAULT_WINDOW_HEIGHT = 600;
const int MIN_WINDOW_WIDTH = 780;
const int MIN_WINDOW_HEIGHT = 585;
const int WINDOW_WIDTH_RATIO = 4;
const int WINDOW_HEIGHT_RATIO = 3;
const bool DEFAULT_FULLSCREEN = false;
const char DEFAULT_WINDOW_TITLE[29] = "Jace's OpenGL Game Framework";
const int KEY_BUFFER_SIZE = 256;
const int FONT_ASCII_VALUE_LIMIT = 128;
const bool DEFAULT_TEXT_CENTERED = false;
const int FRAMES_TO_CALC_AVERAGE_FPS = 20;

////////////////////////////////////////////////////////////////////////////////
// These are the indices of data that is used by more than one game state,
// or a game state and the game framework.
////////////////////////////////////////////////////////////////////////////////
const int SHARED_DATA_COUNT = 5;
const int Fullscreen = 0;
const int DisplayFPS = 1;
const int Difficulty = 2;
const int MissionLevel = 3;
const int LevelSubState = 4;
const int SUB_STATE_PLAYING = 0;
const int SUB_STATE_SUCCESS = 1;
const int SUB_STATE_FAILURE = 2;

////////////////////////////////////////////////////////////////////////////////
// The GameState class is used to divide the game logic into easier to manage
// sections.
////////////////////////////////////////////////////////////////////////////////
class GameState
{
public:
	virtual void Initialize() = 0;
	virtual void CleanUp() = 0;
	virtual void HandleInput(unsigned char key) = 0;
	virtual void Update(double timeDelta, unsigned char *keyBuffer) = 0;
	virtual void Render() = 0;
};

////////////////////////////////////////////////////////////////////////////////
// The GameFramework provides common game functionality to speed development
// time.  Note that it is implemented as a singleton so each state can access
// information to change game state without having to store a specific pointer
// to the global.
////////////////////////////////////////////////////////////////////////////////
class GameFramework
{
public:
	~GameFramework() { mInstance = NULL; };
	static GameFramework *Instance();
	void Initialize(int *argc, char **argv, GameState *state,
					const char *title = DEFAULT_WINDOW_TITLE,
					int width = DEFAULT_WINDOW_WIDTH ,
					int height = DEFAULT_WINDOW_HEIGHT,
					bool fullscreen = DEFAULT_FULLSCREEN);
	void SetState(GameState *state);
	void Shutdown();
	int GetSharedData(int index);
	void SetSharedData(int index, int value);
	void UpdateFullscreenStatus();
	void DrawText(std::string text, float x, float y,
					bool centered = DEFAULT_TEXT_CENTERED);
	void DrawFPS();
protected:
	GameFramework() { mCurrentState = NULL; };
private:
	// singleton pointer
	static GameFramework *mInstance;
	GameState *mCurrentState;
	Timer mTimer;
	unsigned char mKeyBuffer[KEY_BUFFER_SIZE];
	int mTextBase;
	int mSharedData[SHARED_DATA_COUNT];
	int mWidth;
	int mHeight;
	// game functionality
	void LoadSettingsFromFile();
	void WriteSettingsToFile();
	void ValidateSettingsFileData();
	void SetCallbacks();
	void SetUpFont();
	// defined for GLUT
	static void Reshape(int width, int height);
	static void HandleInput(unsigned char key, int x, int y);
	static void KeyUp(unsigned char key, int x, int y);
	static void HandleInputSpecial(int key, int x, int y);
	static void KeyUpSpecial(int key, int x, int y);
	static void Render();
	static void Update();
};

#endif
