////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// particle.cpp
////////////////////////////////////////////////////////////////////////////////

#include "particle.h"

////////////////////////////////////////////////////////////////////////////////
// A Particle is an individual component of a particle system having position,
// velocity, and color.
////////////////////////////////////////////////////////////////////////////////

// set the particle's color
void Particle::setColor(float red, float green, float blue)
{
	mColor.setRed(red);
	mColor.setGreen(green);
	mColor.setBlue(blue);
}

// set the particle's position
void Particle::setPosition(double x, double y, double z)
{
	mPosition.setX(x);
	mPosition.setY(y);
	mPosition.setZ(z);
}

// set the particle's position
void Particle::setVelocity(double x, double y, double z)
{
	mVelocity.setX(x);
	mVelocity.setY(y);
	mVelocity.setZ(z);
}

// set the length of time the particle exists
void Particle::setLifespan(double lifespan)
{
	mDeathClock.SetDelay(lifespan);
}

bool Particle::shouldDie()
{
	return mDeathClock.HasSignaled();
}

// update the particle's position based on its velocity and time elapsed
void Particle::update(double elapsedTime)
{
	mPosition = mPosition + (mVelocity * elapsedTime);
	mDeathClock.Update(elapsedTime);
}

// draw the point corresponding to this particle,
// assuming this is called within a glBegin(GL_POINTS); and glEnd();
void Particle::render()
{
	glColor3f(mColor.getRed(), mColor.getGreen(), mColor.getBlue());
	glVertex3d(mPosition.getX(), mPosition.getY(), mPosition.getZ());
}

////////////////////////////////////////////////////////////////////////////////
// A ParticleSystem manages a collection of particles.
////////////////////////////////////////////////////////////////////////////////

ParticleSystem::~ParticleSystem()
{
	mParticles.erase(mParticles.begin(), mParticles.end());
}

// set the number of particles to create per generation
void ParticleSystem::setSpawnCount(int count)
{
	assert(count > 0);
	mSpawnCount = count;
}

void ParticleSystem::setLifetimeRange(double min, double max)
{
	assert(min > 0);
	assert(max > min);
	mMinLifetime = min;
	mMaxLifetime = max;
}

void ParticleSystem::setVelocityMagnitudeRange(double min, double max)
{
	assert(min > 0);
	assert(max > min);
	mMinMagnitude = min;
	mMaxMagnitude = max;
}

void ParticleSystem::setColorRange(double minRed, double maxRed, double minGreen,
								   double maxGreen, double minBlue, double maxBlue)
{
	mMinColor.setRed(minRed);
	mMinColor.setGreen(minGreen);
	mMinColor.setBlue(minBlue);
	mMaxColor.setRed(maxRed);
	mMaxColor.setGreen(maxGreen);
	mMaxColor.setBlue(maxBlue);
}

//
void ParticleSystem::setGenerationLimit(double delay)
{
	assert(delay >= 0.0);
	mGenerationDelay = delay;
	mGenerationLimit.SetDelay(mGenerationDelay);
}

void ParticleSystem::update(double elapsedTime)
{
	int i = 0;
	std::vector<Particle> tempParticles;

	mGenerationLimit.Update(elapsedTime);

	// for each particle, update its timer so it can know when it should expire
	for(i = 0; i < mParticles.size(); ++i)
	{
		mParticles[i].update(elapsedTime);
	}

	// destroy all particles whose time has expired
	std::vector<Particle>::iterator it;
	for(it = mParticles.begin(); it != mParticles.end(); ++it)
	{
		if(!it->shouldDie())
		{
			tempParticles.push_back(*it);
		}
	}
	mParticles.swap(tempParticles);
	tempParticles.clear();
}

void ParticleSystem::render()
{
	glBegin(GL_POINTS);
		for(int i = 0; i < mParticles.size(); ++i)
		{
			mParticles[i].render();
		}
	glEnd();
}

////////////////////////////////////////////////////////////////////////////////
// Exhaust is a specific effect used for the exhuast of the ship's engines.
////////////////////////////////////////////////////////////////////////////////

//
void Exhaust::setOriginReference(Vector3d *origin)
{
	mOrigin = origin;
}

//
void Exhaust::setVelocityReference(Vector3d * velocity)
{
	mVelocity = velocity;
}

//
void Exhaust::setReferenceAngle(double *angle)
{
	mReferenceAngle = angle;
}

//
void Exhaust::setAngleRange(double angle)
{
	mAngle = angle;
}

// generate particles from a single point, but vary the initial direction
// they move based on the direction of the ship and vary speed based on
// 
void Exhaust::generate()
{
	if(mGenerationLimit.HasSignaled())
	{
		for(int i = 0; i < mSpawnCount; ++i)
		{
			Particle temp;
			double tempOrientation = *mReferenceAngle - PI / 2.0;
			double angle = getInRange(-mAngle - tempOrientation,
										mAngle - tempOrientation);
			double magnitude = getInRange(mMinMagnitude, mMaxMagnitude);
			temp.setPosition(mOrigin->getX() + 2.0 * cos(tempOrientation),
				mOrigin->getY() + 2.0 * sin(tempOrientation), mOrigin->getZ());
			temp.setVelocity(mVelocity->getX() + magnitude * cos(angle),
				mVelocity->getY() + magnitude * -sin(angle), 0.0);
			temp.setLifespan(getInRange(mMinLifetime, mMaxLifetime));
			temp.setColor(getInRange(mMinColor.getRed(), mMaxColor.getRed()),
							getInRange(mMinColor.getGreen(), mMaxColor.getGreen()),
							getInRange(mMinColor.getBlue(), mMaxColor.getBlue()));
			mParticles.insert(mParticles.end(), temp);
		}
		mGenerationLimit.SetDelay(mGenerationDelay);
	}
}
