////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// types.cpp
////////////////////////////////////////////////////////////////////////////////

#include "types.h"

////////////////////////////////////////////////////////////////////////////////
// A Color stores red, green, and blue components of a color and is simple
// enough to be entirely defined in the header file.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// A Vector3d is a three dimensional vectorwith both algebraic and
// vector-specific operations.
////////////////////////////////////////////////////////////////////////////////
Vector3d Vector3d::operator + (const Vector3d &other) const
{
	Vector3d temp;
	temp.setX(vector[X] + other.getX());
	temp.setY(vector[Y] + other.getY());
	temp.setZ(vector[Z] + other.getZ());
	return temp;
}

Vector3d Vector3d::operator - (const Vector3d &other) const
{
	Vector3d temp;
	temp.setX(vector[X] - other.getX());
	temp.setY(vector[Y] - other.getY());
	temp.setZ(vector[Z] - other.getZ());
	return temp;
}

Vector3d Vector3d::operator * (double scalar) const
{
	Vector3d temp;
	temp.setX(vector[X] * scalar);
	temp.setY(vector[Y] * scalar);
	temp.setZ(vector[Z] * scalar);
	return temp;
}

Vector3d operator * (double scalar, const Vector3d &other)
{
	Vector3d temp;
	temp.setX(other.getX() * scalar);
	temp.setY(other.getY() * scalar);
	temp.setZ(other.getZ() * scalar);
	return temp;
}

Vector3d Vector3d::operator / (double scalar) const
{
	Vector3d temp;
	temp.setX(vector[X] / scalar);
	temp.setY(vector[Y] / scalar);
	temp.setZ(vector[Z] / scalar);
	return temp;
}

double Vector3d::magnitude() const
{
	return sqrt(vector[X] * vector[X] + vector[Y] * vector[Y] + vector[Z] * vector[Z]);
}

double Vector3d::dotProduct(const Vector3d &other) const
{
	return vector[X] * other.getX() + vector[Y] * other.getY() + vector[Z] * other.getZ();
}

Vector3d Vector3d::crossProduct(const Vector3d &other) const
{
	Vector3d temp;
	temp.setX(vector[Y] * other.getZ() - vector[Z] * other.getY());
	temp.setY(vector[Z] * other.getX() - vector[X] * other.getZ());
	temp.setZ(vector[X] * other.getY() - vector[Y] * other.getX());
	return temp;
}

// rotates the vector mAngle radians counter-clockwise about origin in the X-Y plane
Vector3d Vector3d::rotateInXY(double mAngle) const
{
	Vector3d temp;
	double currentAngle;

	if(0.001 < vector[X] || vector[X] < -0.001)
	{
		currentAngle = atan(vector[Y] / vector[X]);
	}
	else
	{
		currentAngle = PI / 2.0;
	}

	temp.setX(magnitude() * sin(currentAngle + mAngle));
	temp.setY(-magnitude() * cos(currentAngle + mAngle));
	temp.setZ(0.0);
	return temp;
}


// treat the 3d vectors as 2d by ignoring the z component
double Determinant2d(const Vector3d &a, const Vector3d &b)
{
	return a.getX() * b.getY() - b.getX() * a.getY();
}

// returns true when the line segment with endpoints a1 and a2 
// intersects the line segment with endpoints b1 and b2
bool Intersect(const Vector3d &a1, const Vector3d &a2, const Vector3d &b1, const Vector3d &b2)
{
	Vector3d d1 = a1 - b1;
	Vector3d d2 = b2 - b1;
	Vector3d d3 = a2 - b1;
	Vector3d d4 = b1 - a1;
	Vector3d d5 = a2 - a1;
	Vector3d d6 = b2 - a1;
	return ((Determinant2d(d1, d2) * Determinant2d(d3, d2) < 0) &&
			(Determinant2d(d4, d5) * Determinant2d(d6, d5) < 0));
}

// returns a double randomly chosen within the range of the min and max
double getInRange(double min, double max)
{
	return min + ((max - min) * rand() / RAND_MAX);
}
