////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// countState.h
////////////////////////////////////////////////////////////////////////////////

// Organizational Note:  The various game states are declared in roughly the
// order of appearance as encountered by the player.  State-specific constants
// are declared just prior to corresponding state classes.

#ifndef GAMESTATES_H
#define GAMESTATES_H

#include "framework.h"
#include "audio.h"
#include "level.h"

////////////////////////////////////////////////////////////////////////////////
// StartState ensures the correct fullscreen state is achieved.
////////////////////////////////////////////////////////////////////////////////

class StartState : public GameState
{
public:
	~StartState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static StartState *Instance() { if(0==mInstance){mInstance=new StartState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	StartState() { };
private:
	static StartState *mInstance;
};

////////////////////////////////////////////////////////////////////////////////
// TitleState displays the title screen with the launching spaceship.
////////////////////////////////////////////////////////////////////////////////

// number of stars in the sky on the title screen
const int STAR_COUNT_TITLE = 500;

class TitleState : public GameState
{
public:
	~TitleState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static TitleState *Instance() { if(0==mInstance){mInstance=new TitleState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	TitleState() { };
private:
	void LoadTitleLevel();
	static TitleState *mInstance;
	Ship *mShip;
	Vector3d *mStars;
	Vector3d mLowerLeft;
	Vector3d mUpperRight;
	float fadeText;
	Event mFadeText;
	Event mEndTitle;
	Event mEndZoom;
	Event mIgnition;
};

////////////////////////////////////////////////////////////////////////////////
// IntroState shows the ship in space which fades to the rotating star cube.
////////////////////////////////////////////////////////////////////////////////

// number of stars in the star cube
const int STAR_COUNT_INTRO = 1000;
// arbitrary vertical sped of ship
const double SHIP_CONSTANT_SPEED = 25.0;
// the total time (in seconds) elapsed during initial camera zoom out
const double ZOOM_TIME = 9.0;

class IntroState : public GameState
{
public:
	~IntroState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static IntroState *Instance() { if(0==mInstance){mInstance=new IntroState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	IntroState() { };
private:
	static IntroState *mInstance;
	Vector3d eye;
	Vector3d focus;
	float fadeCube;
	float fadeTextQuest;
	float fadeTextFor;
	float fadeTextThe;
	float mTimeDelta;
	Vector3d *mPoints;
	Event mChangeCameraMovement;
	Event mStartCameraSmooth;
	Event mBlastOffAndFade;
	Event mLightDone;
	Event mTextQuestIn;
	Event mTextForTheIn;
};

////////////////////////////////////////////////////////////////////////////////
// MenuState displays the main menu.
////////////////////////////////////////////////////////////////////////////////

class MenuState : public GameState
{
public:
	~MenuState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static MenuState *Instance() { if(0==mInstance){mInstance=new MenuState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	MenuState() { };
private:
	static MenuState *mInstance;
	Menu mMainMenu;
	float mSelectorHeight;
};

////////////////////////////////////////////////////////////////////////////////
// OptionState displays the options menu.
////////////////////////////////////////////////////////////////////////////////

class OptionState : public GameState
{
public:
	~OptionState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static OptionState *Instance() { if(0==mInstance){mInstance=new OptionState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	OptionState() { };
private:
	static OptionState *mInstance;
	Menu mOptionMenu;
	Option mFullscreen;
	Option mDisplayFPS;
	Option mDifficulty;
	float mSelectorHeight;
	bool mJustChangedFullscreen;
};

////////////////////////////////////////////////////////////////////////////////
// StoryState displays the back story for each level.
////////////////////////////////////////////////////////////////////////////////

class StoryState : public GameState
{
public:
	~StoryState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static StoryState *Instance() { if(0==mInstance){mInstance=new StoryState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	StoryState() { };
private:
	static StoryState *mInstance;
	std::vector<std::string> mStoryLines;
};

////////////////////////////////////////////////////////////////////////////////
// CountState displays a count down until the gameplay begins.
////////////////////////////////////////////////////////////////////////////////

// number of stars in the sky for the credits
const int STAR_COUNT_CREDITS = 1000;

class CountState : public GameState
{
public:
	~CountState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static CountState *Instance() { if(0==mInstance){mInstance=new CountState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	CountState() { };
private:
	static CountState *mInstance;
	int mSecondsElapsed;
	Event mSecond;
};

////////////////////////////////////////////////////////////////////////////////
// Pilot State is where you actually try to land the ship.
////////////////////////////////////////////////////////////////////////////////

// number of stars in the sky on a level
const int STAR_COUNT_PILOT = 500;
// number of levels in game (minus one because base 0)
const int MISSION_COUNT = 3;
// story shown for beating game
const int GAME_BEAT_LEVEL = 8;
// story shown for losing all lives
const int GAME_OVER_LEVEL = 9;

class PilotState : public GameState
{
public:
	~PilotState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static PilotState *Instance() { if(0==mInstance){mInstance=new PilotState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
	void startSubStateTimer();
protected:
	PilotState() { };
private:
	static PilotState *mInstance;
	std::string ConstructFileName();
	Event mLandingStatus;
	Event mNoInstantCollison;
	Vector3d *mStars;
};

////////////////////////////////////////////////////////////////////////////////
// Pause State allows you to continue or quit the current mission.
////////////////////////////////////////////////////////////////////////////////

class PauseState : public GameState
{
public:
	~PauseState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static PauseState *Instance() { if(0==mInstance){mInstance=new PauseState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	PauseState() { };
private:
	static PauseState *mInstance;
	Menu mPauseMenu;
	float mSelectorHeight;
};

////////////////////////////////////////////////////////////////////////////////
// Success State congratulates the player on landing and serves as a break
// between missions.
////////////////////////////////////////////////////////////////////////////////

class SuccessState : public GameState
{
public:
	~SuccessState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static SuccessState *Instance() { if(0==mInstance){mInstance=new SuccessState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	SuccessState() { };
private:
	static SuccessState *mInstance;
};

////////////////////////////////////////////////////////////////////////////////
// CreditsState displays the credits among swirling stars.
////////////////////////////////////////////////////////////////////////////////

// begin fade (in seconds)
const int BEGIN_FADE = 5.0;
// transition fade (in seconds)
const int TRANSITION_FADE = 8.0;

class CreditsState : public GameState
{
public:
	~CreditsState() { mInstance = NULL; };
	// Returns the only instance of the class, creating it if it doesn't exist yet.
	static CreditsState *Instance() { if(0==mInstance){mInstance=new CreditsState;}return mInstance; };
	void Initialize();
	void CleanUp();
	void HandleInput(unsigned char key);
	void Render();
	void Update(double timeDelta, unsigned char *keyBuffer);
protected:
	CreditsState() { };
private:
	static CreditsState *mInstance;
	double mTimeDelta;
	Vector3d *mPoints;
	float mX;
	float mY;
	float mZ;
	float mFade;
	int mCurrentCredit;
	Event mFadeOut;
	Event mNextCredit;
	bool mFadingIn;
	bool mFadingOut;
};

#endif
