////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsIntro.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// IntroState shows the ship in space which fades to the rotating star cube.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
IntroState *IntroState::mInstance = 0;

// Load and play music.
void IntroState::Initialize()
{
	// make background black and enable alpha blending
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// allocate memory for stars, remember to delete
	mPoints = new Vector3d[STAR_COUNT_INTRO];
	for(int i = 0; i < STAR_COUNT_INTRO; i++)
	{
		mPoints[i].setX((double)rand() / RAND_MAX);
		mPoints[i].setY((double)rand() / RAND_MAX);
		mPoints[i].setZ((double)rand() / RAND_MAX);
	}

	// load light and intro introductory music pieces, only light is playing initially
	AudioManager::Instance()->load("Light Theme", "assets\\audio\\light.mp3");
	AudioManager::Instance()->setVolume("Light Theme", 255);
	AudioManager::Instance()->play("Light Theme");
	AudioManager::Instance()->load("Intro Loop", "assets\\audio\\new1.mid");
	AudioManager::Instance()->setVolume("Intro Loop", 80);

	// set the camera in the correct position, looking at the correct spot
	eye.setX(0.5);
	eye.setY(0.75);
	eye.setZ(0.0);
	focus.setX(0.5);
	focus.setY(0.75);
	focus.setZ(0.5);

	// clear view matrices
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)glutGet(GLUT_WINDOW_WIDTH)/glutGet(GLUT_WINDOW_HEIGHT),
					0.1, 100.0);

	// set all text and the containing cube to be transparent at first
	fadeCube = 0.0;
	fadeTextQuest = 0.0;
	fadeTextFor = 0.0;
	fadeTextThe = 0.0;

	// set delays of timed, cinematic events
	mChangeCameraMovement.SetDelay(4.0);
	mStartCameraSmooth.SetDelay(8.0);
	mBlastOffAndFade.SetDelay(9.5);
	mLightDone.SetDelay(12.0);
	mTextQuestIn.SetDelay(2.5);
	mTextForTheIn.SetDelay(6.0);
}

// Any key press goes to the main menu.
void IntroState::HandleInput(unsigned char key)
{
	if(key)
	{
		GameFramework::Instance()->SetState(MenuState::Instance());
	}
}

// Flying camera, spaceship blasting off, title fading in, an overall cool intro.
void IntroState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// Handle 3 camera movements in sequence
	if(!mChangeCameraMovement.HasSignaled())
	{
		// translate the camera down to do fake 2Dish perspective
		// but at the same time suggest 3D world with stars

		// move camera down (from 0.5, 0.75, 0.0 to 0.5, 0.25, 0.0),
		// .1 calculated because desired distance to travel in y is .5
		// and desired time is 5 seconds (.5 / 5 = .1);
		eye.setY(eye.getY() - 0.125 * mTimeDelta);
		// move focus down (from 0.5, 0.75, 0.5 to 0.5, 0.25, 0.5)
		focus.setY(focus.getY() - 0.125 * mTimeDelta);

		gluLookAt(eye.getX(), eye.getY(), eye.getZ(), focus.getX(), focus.getY(),
					focus.getZ(), 0.0, 1.0, 0.0);
	}
	else if(mChangeCameraMovement.HasSignaled() && !mStartCameraSmooth.HasSignaled())
	{
		// rotate from 2Dish view to beginning of star cube reveal

		// move camera forward .25 to get to origin
		//eye.z += 0.0625 * mTimeDelta;
		// move camera down remaining .25 to get to origin for fast movement
		eye.setY(eye.getY() - 0.0625 * mTimeDelta);
		// move camera right (from 0.5, 0.25, 0.0 to 0.0, 0.0, 0.0)
		eye.setX(eye.getX() - 0.125 * mTimeDelta);
		// move focus up (from 0.5, 0.25, 0.5 to 0.5, 0.5, 0.5)
		focus.setY(focus.getY() + 0.0625 * mTimeDelta);

		gluLookAt(eye.getX(), eye.getY(), eye.getZ(), focus.getX(), focus.getY(),
					focus.getZ(), 0.0, 1.0, 0.0);
	}
	else if(mStartCameraSmooth.HasSignaled())
	{
		eye.setX(eye.getX() + 0.064 * mTimeDelta);
		eye.setY(eye.getY() + 0.172 * mTimeDelta);
		eye.setZ(eye.getZ() + 0.128 * mTimeDelta);

		if(eye.getX() >= 2.0 * PI)
			eye.setX(0.0);
		if(eye.getY() >= 2.0 * PI)
			eye.setY(0.0);
		if(eye.getZ() >= 2.0 * PI)
			eye.setZ(0.0);

		gluLookAt(2.5 * sin(eye.getX()), 2.0 * sin(eye.getY()), 2.5 * sin(eye.getZ()),
					0.5, 0.5, 0.5, 0.0, 1.0, 0.0);
	}

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();


	// "Quest"
	if(mTextQuestIn.HasSignaled())
	{
		glColor4f(1.0, 1.0, 1.0, fadeTextQuest);
		GameFramework::Instance()->DrawText("Quest", 0.0, 0.54, true);
		fadeTextQuest += 0.105 * mTimeDelta;
		if(fadeTextQuest > 1.0)
		{
			fadeTextQuest = 1.0;
		}
	}
	// "for the"
	if(mTextForTheIn.HasSignaled())
	{
		glColor4f(1.0, 1.0, 1.0, fadeTextFor);
		GameFramework::Instance()->DrawText("for  the", 0.46, 0.5, true);
		fadeTextFor += 0.105 * mTimeDelta;
		if(fadeTextFor > 1.0)
		{
			fadeTextFor = 1.0;
		}
	}

	// update the alpha for title, ship, and blue box
	if((fadeCube < 1.0) && mBlastOffAndFade.HasSignaled())
	{
		fadeCube += 0.105 * mTimeDelta;
	}

	if(mBlastOffAndFade.HasSignaled())
	{
		glColor4f(1.0, 1.0, 1.0, fadeCube);
		GameFramework::Instance()->DrawText("Star Cube", 0.0, 0.46, true);
	}

	if(mBlastOffAndFade.HasSignaled() || !mChangeCameraMovement.HasSignaled())
	{
		// red fire
		glColor4f(0.8, 0.0, 0.0, 1.0 - 1.8 * fadeCube);
		glBegin(GL_LINE_STRIP);
			glVertex3f(0.5033, 0.475 + fadeCube, 0.5);
			glVertex3f(0.5, 0.466 + fadeCube, 0.5);
			glVertex3f(0.4967, 0.475 + fadeCube, 0.5);
		glEnd();
	}

	// gray ship
	glColor4f(0.3, 0.3, 0.3, 1.0 - 1.8 * fadeCube);
	glBegin(GL_TRIANGLES);
		glVertex3f(0.5, 0.5 + fadeCube, 0.5);
		glVertex3f(0.51, 0.475 + fadeCube, 0.5);
		glVertex3f(0.49, 0.475 + fadeCube, 0.5);
	glEnd();
	// green outline of ship
	glColor4f(0.7, 0.7, 0.7, 1.0 - 1.8 * fadeCube);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.5, 0.5 + fadeCube, 0.5);
		glVertex3f(0.51, 0.475 + fadeCube, 0.5);
		glVertex3f(0.49, 0.475 + fadeCube, 0.5);
	glEnd();

	// blue cube
	glColor4f(0.0, 0.0, 0.5, 3.0 * fadeCube);
	glBegin(GL_LINE_STRIP);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(1.0, 0.0, 0.0);
		glVertex3f(1.0, 1.0, 0.0);
		glVertex3f(0.0, 1.0, 0.0);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(0.0, 0.0, 1.0);
		glVertex3f(1.0, 0.0, 1.0);
		glVertex3f(1.0, 1.0, 1.0);
		glVertex3f(0.0, 1.0, 1.0);
		glVertex3f(0.0, 0.0, 1.0);
	glEnd();
	glBegin(GL_LINES);
		glVertex3f(1.0, 0.0, 0.0);
		glVertex3f(1.0, 0.0, 1.0);
		glVertex3f(1.0, 1.0, 0.0);
		glVertex3f(1.0, 1.0, 1.0);
		glVertex3f(0.0, 1.0, 0.0);
		glVertex3f(0.0, 1.0, 1.0);
	glEnd();

	// white stars
	glColor4f(1.0, 1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
		for(int i = 0; i < STAR_COUNT_INTRO; i++)
		{
			glVertex3f(mPoints[i].getX(), mPoints[i].getY(), mPoints[i].getZ());
		}
	glEnd();

	// update currently playing music to play in sequence thx, light, then intro
	if(mLightDone.HasSignaled())
	{
		AudioManager::Instance()->stop("Light Theme");
		AudioManager::Instance()->play("Intro Loop");
		mLightDone.SetDelay(100000.0);
	}
}

// 
void IntroState::Update(double timeDelta, unsigned char *keyBuffer)
{
	mTimeDelta = timeDelta;
	mLightDone.Update(timeDelta);
	mChangeCameraMovement.Update(timeDelta);
	mBlastOffAndFade.Update(timeDelta);
	mStartCameraSmooth.Update(timeDelta);
	mTextQuestIn.Update(timeDelta);
	mTextForTheIn.Update(timeDelta);
	// new particle system
	Ship::Instance()->igniteEngines();
	Ship::Instance()->setPosition(Vector3d(0.5, 0.45 + fadeCube, 0.4));
	Ship::Instance()->SimulateParticles(timeDelta);
}

// Free the loaded music.
void IntroState::CleanUp()
{
	AudioManager::Instance()->unload("Light Theme");
	AudioManager::Instance()->unload("Intro Loop");
	delete [] mPoints;
}
