////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// gsPilot.cpp
////////////////////////////////////////////////////////////////////////////////

#include "gameStates.h"

////////////////////////////////////////////////////////////////////////////////
// Pilot State is where you actually try to land the ship.
////////////////////////////////////////////////////////////////////////////////

// Declare the single instance.
PilotState *PilotState::mInstance = 0;

// Load level from file.
void PilotState::Initialize()
{
	// load the music
	AudioManager::Instance()->load("Mission", "assets\\audio\\mission.mid");
	AudioManager::Instance()->setVolume("Mission", 110);
	AudioManager::Instance()->load("Failure", "assets\\audio\\failure.mp3");
	AudioManager::Instance()->setVolume("Failure", 160);
	AudioManager::Instance()->load("Success", "assets\\audio\\success.mp3");
	AudioManager::Instance()->setVolume("Success", 160);

	Level::Instance()->LoadFromFile(ConstructFileName());
	GameFramework::Instance()->SetSharedData(LevelSubState, SUB_STATE_PLAYING);

	if(AudioManager::Instance()->isPaused("Mission"))
	{
		AudioManager::Instance()->resume("Mission");
	}
	else
	{
		AudioManager::Instance()->play("Mission");
	}

	// allocate memory for stars, remember to delete
	mStars = new Vector3d[STAR_COUNT_PILOT];

	// position stars so they appear above the horizon
	for(int i = 0; i < STAR_COUNT_PILOT; ++i)
	{
		mStars[i].setX((rand() % GAME_SCREEN_WIDTH));
		mStars[i].setY((rand() % GAME_SCREEN_HEIGHT));
		mStars[i].setZ(0.0);
	}

	mLandingStatus.SetDelay(10.0);
	mNoInstantCollison.SetDelay(1.2); // this timer is used to 

	glClearColor(0.0, 0.0, 0.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, GAME_SCREEN_WIDTH, 0.0, GAME_SCREEN_HEIGHT);
}

// Only input besides ship control is to pause the game.
// Arrow keys are handled in MainGameState::Update()
void PilotState::HandleInput(unsigned char key)
{
	if(SUB_STATE_PLAYING == GameFramework::Instance()->GetSharedData(LevelSubState))
	{
		// ESCAPE, ENTER or SPACE will pause gameplay
		if((27 == key) || (13 == key) || (' ' == key))
		{
			AudioManager::Instance()->pause("Mission");
			GameFramework::Instance()->SetState(PauseState::Instance());
		}
	}
}

// Draw the ship, level, fuel meter, etc.
void PilotState::Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// draw stars in the sky
	glColor4f(1.0, 1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
		for(int i = 0; i < STAR_COUNT_PILOT; i++)
		{
			glVertex3d(mStars[i].getX(), mStars[i].getY(), 0.0);
		}
	glEnd();

	// draw the level, stats, and ship
	Level::Instance()->Render();
	Ship::Instance()->Render(2.0);
	Level::Instance()->RenderOverlayStats();

	switch(GameFramework::Instance()->GetSharedData(LevelSubState))
	{
	case SUB_STATE_SUCCESS:
		// Draw "SUCCESSFUL LANDING!"
		glColor4f(0.4, 0.6, 0.0, 1.0);
		GameFramework::Instance()->DrawText("SUCCESSFUL LANDING!", 0.0, 0.6, true);
		break;
	case SUB_STATE_FAILURE:
		// Draw "CRASH LANDING!"
		glColor4f(1.0, 0.0, 0.0, 1.0);
		GameFramework::Instance()->DrawText("CRASH LANDING!", 0.0, 0.6, true);
		break;
	}

	glColor3f(1.0, 1.0, 1.0);
	GameFramework::Instance()->DrawFPS();
}

// Check for collisions between each edge of the ship and each edge of the level.
// if thrusting, use some of the fuel
// update velocity based on acceleration
// update position based on velocity
// update arrow key presses
void PilotState::Update(double timeDelta, unsigned char *keyBuffer)
{
	mNoInstantCollison.Update(timeDelta);
	switch(GameFramework::Instance()->GetSharedData(LevelSubState))
	{
	case SUB_STATE_PLAYING:
		Ship::Instance()->Simulate(timeDelta, keyBuffer);
		Level::Instance()->ZoomCamera();
		if(mNoInstantCollison.HasSignaled())
		{
			Level::Instance()->CheckCollisions();
		}
		break;
	case SUB_STATE_SUCCESS:
	case SUB_STATE_FAILURE:
		// update landing timer
		mLandingStatus.Update(timeDelta);
		break;
	}

	// when the landing timer expires, load the countdown screen
	if(mLandingStatus.HasSignaled())
	{
		if(SUB_STATE_FAILURE == GameFramework::Instance()->GetSharedData(LevelSubState))
		{
			if(0 > Ship::Instance()->getLives())
			{
				// game over
				GameFramework::Instance()->SetSharedData(MissionLevel, GAME_OVER_LEVEL);
				GameFramework::Instance()->SetState(StoryState::Instance());
			}
			else
			{
				// restart level
				GameFramework::Instance()->SetState(CountState::Instance());
			}
		}
		if(SUB_STATE_SUCCESS == GameFramework::Instance()->GetSharedData(LevelSubState))
		{
			if((MISSION_COUNT + 1) == GameFramework::Instance()->GetSharedData(MissionLevel))
			{
				// won game
				GameFramework::Instance()->SetSharedData(MissionLevel, GAME_BEAT_LEVEL);
				GameFramework::Instance()->SetState(StoryState::Instance());
			}
			else
			{
				// advance level
				GameFramework::Instance()->SetState(StoryState::Instance());
			}
		}
	}
}

// free loaded music
void PilotState::CleanUp()
{
	AudioManager::Instance()->unload("Success");
	AudioManager::Instance()->unload("Failure");
	delete [] mStars;
}

// Returns the name of the file to load that corresponds to the current mission.
std::string PilotState::ConstructFileName()
{
	// calculate the character corresponding to level number ('0' is ASCII 48)
	unsigned char levelNumber = 48 + GameFramework::Instance()->GetSharedData(MissionLevel);

	// assemble string of file to load
	std::string fileName = "assets\\text\\mission";
	fileName += levelNumber;
	fileName.append(".txt");

	return fileName;
}

void PilotState::startSubStateTimer()
{
	mLandingStatus.SetDelay(6.0);
}
