////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// level.cpp
////////////////////////////////////////////////////////////////////////////////

#include "level.h"

////////////////////////////////////////////////////////////////////////////////
// A Level consists of terrain made up of line segments, a landing pad,
// and a ship.  Levels can be loaded from file.
////////////////////////////////////////////////////////////////////////////////

// Loads information about a level.
// Also used to load the title screen.
void Level::LoadFromFile(std::string fileName)
{
	std::fstream levelFile;
	double tempX, tempY, tempOrientation;

	glClearStencil(0);

	levelFile.open(fileName.c_str(), std::ios::in);

	mTerrain.clear();
	mTerrain.resize(0);
	mPolyBreaks.clear();
	mPolyBreaks.resize(0);
	if(levelFile.is_open())
	{
		// read in the ships starting position
		levelFile >> tempX;
		levelFile >> tempY;
		
		// read in the ship's initial orientation (angle in radians from vertical)
		levelFile >> tempOrientation;

		if(SUB_STATE_PLAYING == GameFramework::Instance()->GetSharedData(LevelSubState))
		{
			if(!AudioManager::Instance()->isPaused("Mission"))
			{
				Ship::Instance()->setPosition(Vector3d(tempX, tempY, 0.0));
				Ship::Instance()->setOrientation(tempOrientation);
				Ship::Instance()->setVelocity(Vector3d(0.0, 0.0, 0.0));
				Ship::Instance()->setFuel(1.0);
				// Begin Nautilus level with 70% fuel
				if(MISSION_COUNT == GameFramework::Instance()->GetSharedData(MissionLevel))
				{
					Ship::Instance()->setFuel(0.70);
				}
			}
		}
		else
		{
			Ship::Instance()->setPosition(Vector3d(tempX, tempY, 0.0));
			Ship::Instance()->setOrientation(tempOrientation);
			Ship::Instance()->setVelocity(Vector3d(0.0, 0.0, 0.0));
			Ship::Instance()->setFuel(1.0);
			// Begin Nautilus level with 70% fuel
			if(MISSION_COUNT == GameFramework::Instance()->GetSharedData(MissionLevel))
			{
				Ship::Instance()->setFuel(0.70);
			}
		}

		// read in the landing platform's position
		levelFile >> tempX;
		levelFile >> tempY;
		mPlatform[3].setX(tempX);
		mPlatform[3].setY(tempY);
		// go ahead and set the platform size since it doesn't change in most levels
		mPlatform[0].setX(mPlatform[3].getX() + PLATFORM_WIDTH);
		mPlatform[0].setY(mPlatform[3].getY());
		mPlatform[1].setX(mPlatform[3].getX() + PLATFORM_WIDTH);
		mPlatform[1].setY(mPlatform[3].getY() + PLATFORM_HEIGHT);
		mPlatform[2].setX(mPlatform[3].getX());
		mPlatform[2].setY(mPlatform[3].getY() + PLATFORM_HEIGHT);

		// read in number of (possibly) concave polygons, no more than MAX_POLY_COUNT
		levelFile >> mConcavePolyCount;
		assert(mConcavePolyCount <= MAX_POLY_COUNT);

		for(int i = 0; i < mConcavePolyCount; ++i)
		{
			levelFile >> mPointsPerPoly[i];
			if(0 == i)
			{
				mPolyBreaks.push_back(mPointsPerPoly[0]);
			}
			else
			{
				mPolyBreaks.push_back(mPolyBreaks[i - 1] + mPointsPerPoly[i]);
			}
			for(int j = 0; j < mPointsPerPoly[i]; ++j)
			{
				// read points of each polygon
				levelFile >> tempX >> tempY;
				mTerrain.push_back(Vector3d(tempX, tempY, 0.0));
			}
		}
		levelFile.close();
		printf("Number of points in file: %d", mTerrain.size());
	}
}

// 
void CrashShip()
{
	// signal to change to substate of PilotState: Crash
	GameFramework::Instance()->SetSharedData(LevelSubState, SUB_STATE_FAILURE);
	// remove one life
	Ship::Instance()->crash();
	// set landing timer
	PilotState::Instance()->startSubStateTimer();
	// play failure music
	if(!AudioManager::Instance()->isPaused("Mission"))
	{
		AudioManager::Instance()->pause("Mission");
	}
	AudioManager::Instance()->play("Failure");
}

// Check for collisions between ship and terrain (crashes) and platform (landings).
void Level::CheckCollisions()
{
	Vector3d bow = Ship::Instance()->getBow();
	Vector3d port = Ship::Instance()->getPort();
	Vector3d starboard = Ship::Instance()->getStarboard();
	Vector3d position = Ship::Instance()->getPosition();
	int polyBreakIndex = 0;

	// check for collisions between ship and terrain (automatically crashes)
	for(int i = 0; i < mTerrain.size(); ++i)
	{
		int j = (i + 1) % mTerrain.size();
		if(j == mPolyBreaks[polyBreakIndex])
		{
			if(0 == polyBreakIndex)
			{
				j = 0;
			}
			else
			{
				j = mPolyBreaks[polyBreakIndex - 1];
			}
			++polyBreakIndex;
		}
		if(Intersect(bow, port, mTerrain[i], mTerrain[j]) ||
			Intersect(port, starboard, mTerrain[i], mTerrain[j]) ||
			Intersect(starboard, bow, mTerrain[i], mTerrain[j]))
		{
			CrashShip();
			return;
		}
	}

	// check for collisions between ship and top 3 edges of platform
	// if the ship is oriented nearly vertical, moving slowly, above the platform,
	// and the extremes of the ship are between the extremes of the platform,
	// then the landing is good
	for(int j = 0; j < 3; ++j)
	{
		// if collision occurs between any side of ship and platform
		if(Intersect(bow, port, mPlatform[j], mPlatform[j+1]) ||
			Intersect(port, starboard, mPlatform[j], mPlatform[j+1]) ||
			Intersect(starboard, bow, mPlatform[j], mPlatform[j+1]))
		{
			// ship position between left and right ends of platform
			if((position.getX() > (mPlatform[2].getX() + HORIZONTAL_VARIANCE)) &&
				(position.getX() < (mPlatform[0].getX() - HORIZONTAL_VARIANCE)))
			{
				// if the ship is above the platform
				if(position.getY() > mPlatform[2].getY())
				{
					// if the orientation is close to vertical
					if((Ship::Instance()->getOrientation() < ROTATIONAL_VARIANCE) &&
						(Ship::Instance()->getOrientation() > -ROTATIONAL_VARIANCE))
					{
						// if the speed is slow enough and not too fast vertical
						if(Ship::Instance()->getVelocity().magnitude() < MAXIMUM_LANDING_SPEED &&
							Ship::Instance()->getVelocity().getY() > -MAXIMUM_LANDING_SPEED / 2.0)
						{
							// set successful landing state
							GameFramework::Instance()->SetSharedData(LevelSubState,
																		SUB_STATE_SUCCESS);
							// change current level
							GameFramework::Instance()->SetSharedData(MissionLevel,
								(GameFramework::Instance()->GetSharedData(MissionLevel)+1));
							// set landing timer
							PilotState::Instance()->startSubStateTimer();
							// play success music
							if(!AudioManager::Instance()->isPaused("Mission"))
							{
								AudioManager::Instance()->pause("Mission");
							}
							AudioManager::Instance()->play("Success");
							return;
						}
						// crash, too fast
					}
					// crash, not level enough
				}
				// crash, ship below platform
			}
			// crash, ship to side of platform
			CrashShip();
			return;
		}
	}
}

// Zoom the camera to contain the ship and the landing platform.
void Level::ZoomCamera()
{
	// distance between ship and platform corresponds to zoom level
	// if view with ship centered outside the world rectangle, shift the
	// center of the view off of the ship but keep the view size the same.

	Vector3d position = Ship::Instance()->getPosition();
	float zoomLevel = (position - (mPlatform[3] + Vector3d(10, 2.5, 0.0))).magnitude() + 100.0;
	Vector3d centerToCorner = Vector3d(4.0, 3.0, 0.0) * zoomLevel / 6.4; // 6.4 sets speed of zooming
	Vector3d lowerLeft, upperRight;
	
	// change position from being the ship's position to being half way between ship and platform
	position = position + ((mPlatform[3] + Vector3d(10, 2.5, 0.0)) - position) / 3.0;

	// if the whole world is visible,
	// forego calculations to ensure that space outside world is not visible
	if(centerToCorner.getX() < (GAME_SCREEN_WIDTH / 2.0))
	{
		lowerLeft = position - centerToCorner;
		upperRight = position + centerToCorner;

		if(lowerLeft.getX() < 0.0)
		{
			upperRight.setX(upperRight.getX() - lowerLeft.getX());
			lowerLeft.setX(0.0);
		}
		if(lowerLeft.getY() < 0.0)
		{
			upperRight.setY(upperRight.getY() - lowerLeft.getY());
			lowerLeft.setY(0.0);
		}
		if(upperRight.getX() > GAME_SCREEN_WIDTH)
		{
			lowerLeft.setX(lowerLeft.getX() + GAME_SCREEN_WIDTH - upperRight.getX());
			upperRight.setX(GAME_SCREEN_WIDTH);
		}
		if(upperRight.getY() > GAME_SCREEN_HEIGHT)
		{
			lowerLeft.setY(lowerLeft.getY() + GAME_SCREEN_HEIGHT - upperRight.getY());
			upperRight.setY(GAME_SCREEN_HEIGHT);
		}
	}
	else
	{
		lowerLeft.setX(0.0);
		lowerLeft.setY(0.0);
		upperRight.setX(GAME_SCREEN_WIDTH);
		upperRight.setY(GAME_SCREEN_HEIGHT);
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(lowerLeft.getX(), upperRight.getX(), lowerLeft.getY(), upperRight.getY());
}

// Draws the terrain and platform that make up the level, then 
void Level::Render()
{
	// draw filled, concave polygons by using the stencil buffer
	glClear(GL_STENCIL_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// disable color buffer
	glColorMask(0.0, 0.0, 0.0, 0.0);

	// Set the appropriate action for the stencil buffer.  In this case, inverting
	// the value at each point when drawn on will toggle the value of each pixel
	// each time it is drawn on.  The concave filling algorithm says to only count
	// pixels as part of the concave polygon that have been drawn over an odd
	// number of times.

	/////////////////////////////////////////////////////////////////////////
	// invert stencil buffer the correct number of times only where the depth buffer fails
	/////////////////////////////////////////////////////////////////////////
	glStencilFunc(GL_ALWAYS, 0x1, 0x1);
	glStencilOp(GL_INVERT, GL_INVERT, GL_INVERT);

	// for each concave polygon
	int j = 0;
	int k = 0;
	for(int i = 0; i < mConcavePolyCount; ++i)
	{
		k += mPointsPerPoly[i];
		glBegin(GL_TRIANGLE_FAN);
			glVertex2d(0.0, 0.0);
			// for each constituent triangle
			for(; j < k; ++j)
			{
				glVertex2d(mTerrain[j].getX(), mTerrain[j].getY());
			}
			glVertex2d(mTerrain[k - mPointsPerPoly[i]].getX(),
						mTerrain[k - mPointsPerPoly[i]].getY());
		glEnd();
	}

	// enable color buffer
	glColorMask(1.0, 1.0, 1.0, 1.0);

	// draw dark green filled poly
	glColor3f(0.0, 0.2, 0.0);
	glStencilFunc(GL_LEQUAL, 0x1, 0x1);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
	j = 0;
	k = 0;
	for(int i = 0; i < mConcavePolyCount; ++i)
	{
		k += mPointsPerPoly[i];
		glBegin(GL_TRIANGLE_FAN);
			glVertex2d(0.0, 0.0);
			for(; j < k; ++j)
			{
				glVertex2d(mTerrain[j].getX(), mTerrain[j].getY());
			}
			glVertex2d(mTerrain[k - mPointsPerPoly[i]].getX(),
						mTerrain[k - mPointsPerPoly[i]].getY());
		glEnd();
	}

	// light green outline around polygon
	glStencilFunc(GL_ALWAYS, 0x1, 0x1);
	glStencilOp(GL_INVERT, GL_INVERT, GL_INVERT);
	// for each concave polygon
	glColor3f(0.0, 0.8, 0.0);
	j = 0;
	k = 0;
	for(int i = 0; i < mConcavePolyCount; ++i)
	{
		k += mPointsPerPoly[i];
		// draw outline of polygon
		glBegin(GL_LINE_LOOP);
			for(; j < k; ++j)
			{
				glVertex2d(mTerrain[j].getX(), mTerrain[j].getY());
			}
		glEnd();
	}

	// draw Platform
	glColor3f(0.0, 0.3, 0.3);
	glBegin(GL_TRIANGLE_FAN);
		glVertex3f(mPlatform[0].getX(), mPlatform[0].getY(), 0.0);
		glVertex3f(mPlatform[1].getX(), mPlatform[1].getY(), 0.0);
		glVertex3f(mPlatform[2].getX(), mPlatform[2].getY(), 0.0);
		glVertex3f(mPlatform[3].getX(), mPlatform[3].getY(), 0.0);
	glEnd();
	glColor3f(0.0, 0.8, 0.8);
	glBegin(GL_LINE_LOOP);
		glVertex3f(mPlatform[0].getX(), mPlatform[0].getY(), 0.0);
		glVertex3f(mPlatform[1].getX(), mPlatform[1].getY(), 0.0);
		glVertex3f(mPlatform[2].getX(), mPlatform[2].getY(), 0.0);
		glVertex3f(mPlatform[3].getX(), mPlatform[3].getY(), 0.0);
	glEnd();
}

// Render the fuel gauge, altitude meter, and stock, the positions of which are
// not affected by camera zoom.
void Level::RenderOverlayStats()
{
	double fuelLevel = Ship::Instance()->getFuel();

	// vector from lower left corner of screen to upper left corner of fuel gauge
	Vector3d fuelGaugeOffset(5.0, 295.0, 0.0);
	// vector from upper left to lower right of fuel gauge
	Vector3d fuelGaugeDimensions(100.0, -5.0, 0.0);
	// vector from lower left corner of screen to center of rightmost stock ship
	Vector3d stockOffset(393.0, 7.0, 0.0);
	// vector from the center of one stock ship to the next one (to the left)
	Vector3d stockSpacing(-10.0, 0.0, 0.0);

	// ensure the coordinates for the overlay display are always 400x300
	// and save the zooming camera view to restore later
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0.0, GAME_SCREEN_WIDTH, 0.0, GAME_SCREEN_HEIGHT);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	// draw green border around screen edges
	glColor3f(0.0, 1.0, 0.0);
	glLineWidth(10.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(400.0, 0.0, 0.0);
		glVertex3f(400.0, 300.0, 0.0);
		glVertex3f(0.0, 300.0, 0.0);
	glEnd();
	glColor3f(0.0, 0.0, 0.0);
	glLineWidth(8.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(400.0, 0.0, 0.0);
		glVertex3f(400.0, 300.0, 0.0);
		glVertex3f(0.0, 300.0, 0.0);
	glEnd();
	glLineWidth(1.0);

	// draw black background inside fuel gauge
	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_QUADS);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY() +
					fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY() + fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY(), 0.0);
	glEnd();
	// draw box around fuel gauge
	glColor3f(0.0, 1.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY() +
					fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY() + fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY(), 0.0);
	glEnd();

	// change color of fuel meter when fuel gets low (yellow below 33% and red below 10%)
	if(fuelLevel < 0.33)
	{
		glColor3f(1.0, 1.0, 0.0);
	}
	if(fuelLevel < 0.1)
	{
		glColor3f(1.0, 0.0, 0.0);
	}

	// draw the remaining fuel level
	glBegin(GL_QUADS);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX(), fuelGaugeOffset.getY() +
					fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelLevel * fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY() + fuelGaugeDimensions.getY(), 0.0);
		glVertex3f(fuelGaugeOffset.getX() + fuelLevel * fuelGaugeDimensions.getX(),
					fuelGaugeOffset.getY(), 0.0);
	glEnd();

	// draw Stock in lower right corner, keeping outline dimensions proportional to ship
	for(int i = 0; i < Ship::Instance()->getLives(); ++i)
	{
		Vector3d stockShipPosition = stockOffset + (stockSpacing * static_cast<float>(i));
		glColor3f(1.0, 1.0, 0.0);
		glBegin(GL_LINE_LOOP);
			glVertex3f(stockShipPosition.getX(), stockShipPosition.getY() + 4.5, 0.0);
			glVertex3f(stockShipPosition.getX() + 3.0, stockShipPosition.getY() - 3.0, 0.0);
			glVertex3f(stockShipPosition.getX() - 3.0, stockShipPosition.getY() - 3.0, 0.0);
		glEnd();
	}

	// restore previous view
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
}
