////////////////////////////////////////////////////////////////////////////////
// Ace Lander Project
// Jace Miller
// ship.cpp
////////////////////////////////////////////////////////////////////////////////

#include "ship.h"

// Sets up the ship member variables so none are uninitialized.
void Ship::Initialize()
{
	// The ship is initially at rest.
	mVelocity = Vector3d(0.0, 0.0, 0.0);
	// has a full tank of gas,
	mFuel = 1.0;

	// thrust acts in the direction the ship is facing
	mThrust = Vector3d(29.0 - 0.35 * GameFramework::Instance()->GetSharedData(Difficulty), 0.0, 0.0);

	// gravity generally acts downward toward the planet
	mGravity = Vector3d(0.0, -6.0 - 0.3 * GameFramework::Instance()->GetSharedData(Difficulty), 0.0);

	// The number of lives in stock ranges from 3 to 5,
	// and is decreases as difficulty level increases.
	mLives = 3 + (2 - GameFramework::Instance()->GetSharedData(Difficulty));

	// set up exhaust particle system
	mExhaust.setVelocityMagnitudeRange(0.1, 7.0);
	mExhaust.setColorRange(1.0, 1.0, 0.0, 0.8, 0.0, 0.0);
	mExhaust.setLifetimeRange(0.1, 0.2);
	mExhaust.setSpawnCount(20);
	// exhaust specific initialization
	mExhaust.setOriginReference(&mPosition);
	mExhaust.setVelocityReference(&mVelocity);
	mExhaust.setAngleRange(1.5);
	mExhaust.setReferenceAngle(&mOrientation);
	mExhaust.setGenerationLimit(0.01);
}

// Handle input (thrusting and turning), simulate gravity, and update ship position.
void Ship::Simulate(double timeDelta, unsigned char *keyBuffer)
{
	// Left and right arrow keys rotate the ship counter-clockwise and clockwise, respectively
	if(keyBuffer[GLUT_KEY_LEFT])
	{
		setOrientation(mOrientation + ROTATION_RATE * timeDelta);
	}
	if(keyBuffer[GLUT_KEY_RIGHT])
	{
		setOrientation(mOrientation - ROTATION_RATE * timeDelta);
	}

	// Up arrow key applies thrust from engines if there is sufficient fuel
	if(keyBuffer[GLUT_KEY_UP] && mFuel > 0.0)
	{
		// apply effects of thrusting to ship's velocity
		mVelocity = mVelocity - mThrust.rotateInXY(mOrientation) * timeDelta;

		// firing engine consumes fuel
		mFuel -= FUEL_CONSUMPTION_RATE * timeDelta;

		igniteEngines();
	}

	// apply effects of gravity to ship's velocity
	mVelocity = mVelocity + (mGravity * timeDelta);

	// update ship's position based on velocity
	mPosition = mPosition + (mVelocity * timeDelta);

	// update particles
	mExhaust.update(timeDelta);
}

// Draw the ship and the exhaust flame.
void Ship::Render(double scale, double z)
{
	// The proportions of the ship are 4 wide by 5 high.  If the center of the ship,
	// denoted by mPosition were at the origin and the orientation was 0, then the
	// bow vertex would be located at (0.0, 3.0), the starboard vertex at (2.0, -2.0),
	// and the port vertex at (-2.0, -2.0).  The flame of the exhaust has the same
	// proportions as the ship, but is scaled to 3/5 the size of the ship.
	
	// update ship vertices based on position (center of mass)
	mBow = mPosition + Vector3d(-1.5 * sin(mOrientation), 1.5 * cos(mOrientation), 0.0) * scale;
	mPort = mPosition + (PORT.rotateInXY(mOrientation) * scale);
	mStarboard = mPosition + (STARBOARD.rotateInXY(mOrientation) * scale);

	// draw ship
	glColor3f(0.3, 0.3, 0.3);
	glBegin(GL_TRIANGLES);
		glVertex3d(mBow.getX(), mBow.getY(), z);
		glVertex3d(mPort.getX(), mPort.getY(), z);
		glVertex3d(mStarboard.getX(), mStarboard.getY(), z);
	glEnd();
	// draw outline around ship
	glColor3f(0.7, 0.7, 0.7);
	glBegin(GL_LINE_LOOP);
		glVertex3d(mBow.getX(), mBow.getY(), z);
		glVertex3d(mPort.getX(), mPort.getY(), z);
		glVertex3d(mStarboard.getX(), mStarboard.getY(), z);
	glEnd();

	// draw particles
	mExhaust.render();
}

void Ship::setOrientation(double newOrientation)
{
	mOrientation = newOrientation;
	if(mOrientation < -PI)
	{
		mOrientation += 2.0 * PI;
	}
	if(mOrientation > PI)
	{
		mOrientation -= 2.0 * PI;
	}
}

void Ship::setFuel(double fuel)
{
	mFuel = fuel;
}

void Ship::setPosition(Vector3d position)
{
	mPosition = position;
}

void Ship::setVelocity(Vector3d velocity)
{
	mVelocity = velocity;
}

const Vector3d &Ship::getBow() const
{
	return mBow;
}

const Vector3d &Ship::getPort() const
{
	return mPort;
}

const Vector3d &Ship::getStarboard() const
{
	return mStarboard;
}

const Vector3d &Ship::getPosition() const
{
	return mPosition;
}

const Vector3d &Ship::getVelocity() const
{
	return mVelocity;
}

double Ship::getOrientation() const
{
	return mOrientation;
}

double Ship::getFuel() const
{
	return mFuel;
}

int Ship::getLives() const
{
	return mLives;
}

void Ship::igniteEngines()
{
	mExhaust.generate();
}

void Ship::crash()
{
	--mLives;
}

void Ship::SimulateParticles(double timeDelta)
{
	// update particles
	mExhaust.update(timeDelta);
}
